@@ -757,6 +757,14 @@ SCREEN SHOTS
 ==============================================================================
 */
 
+static void SCR_ScreenShot_Usage (void)
+{
+	Con_Printf ("usage: screenshot <format> <quality>\n");
+	Con_Printf ("   format must be \"png\" or \"tga\" or \"jpg\"\n");
+	Con_Printf ("   quality must be 1-100\n");
+	return;
+}
+
 /*
 ==================
 SCR_ScreenShot_f -- johnfitz -- rewritten to use Image_WriteTGA
@@ -765,15 +773,44 @@ SCR_ScreenShot_f -- johnfitz -- rewritten to use Image_WriteTGA
 void SCR_ScreenShot_f (void)
 {
 	byte	*buffer;
-	char	tganame[16];  //johnfitz -- was [80]
+	char	ext[4];
+	char	imagename[16];  //johnfitz -- was [80]
 	char	checkname[MAX_OSPATH];
-	int	i;
+	int	i, quality;
+	qboolean	ok;
 
+	Q_strncpy (ext, "png", sizeof(ext));
+
+	if (Cmd_Argc () >= 2)
+	{
+		const char	*requested_ext = Cmd_Argv (1);
+
+		if (!q_strcasecmp ("png", requested_ext)
+		    || !q_strcasecmp ("tga", requested_ext)
+		    || !q_strcasecmp ("jpg", requested_ext))
+			Q_strncpy (ext, requested_ext, sizeof(ext));
+		else
+		{
+			SCR_ScreenShot_Usage ();
+			return;
+		}
+	}
+
+// read quality as the 3rd param (only used for JPG)
+	quality = 90;
+	if (Cmd_Argc () >= 3)
+		quality = Q_atoi (Cmd_Argv(2));
+	if (quality < 1 || quality > 100)
+	{
+		SCR_ScreenShot_Usage ();
+		return;
+	}
+	
 // find a file name to save it to
 	for (i=0; i<10000; i++)
 	{
-		q_snprintf (tganame, sizeof(tganame), "spasm%04i.tga", i);	// "fitz%04i.tga"
-		q_snprintf (checkname, sizeof(checkname), "%s/%s", com_gamedir, tganame);
+		q_snprintf (imagename, sizeof(imagename), "spasm%04i.%s", i, ext);	// "fitz%04i.tga"
+		q_snprintf (checkname, sizeof(checkname), "%s/%s", com_gamedir, imagename);
 		if (Sys_FileTime(checkname) == -1)
 			break;	// file doesn't exist
 	}
@@ -794,10 +831,19 @@ void SCR_ScreenShot_f (void)
 	glReadPixels (glx, gly, glwidth, glheight, GL_RGB, GL_UNSIGNED_BYTE, buffer);
 
 // now write the file
-	if (Image_WriteTGA (tganame, buffer, glwidth, glheight, 24, false))
-		Con_Printf ("Wrote %s\n", tganame);
+	if (!q_strncasecmp (ext, "png", sizeof(ext)))
+		ok = Image_WritePNG (imagename, buffer, glwidth, glheight, 24, false);
+	else if (!q_strncasecmp (ext, "tga", sizeof(ext)))
+		ok = Image_WriteTGA (imagename, buffer, glwidth, glheight, 24, false);
+	else if (!q_strncasecmp (ext, "jpg", sizeof(ext)))
+		ok = Image_WriteJPG (imagename, buffer, glwidth, glheight, 24, quality, false);
 	else
-		Con_Printf ("SCR_ScreenShot_f: Couldn't create a TGA file\n");
+		ok = false;
+
+	if (ok)
+		Con_Printf ("Wrote %s\n", imagename);
+	else
+		Con_Printf ("SCR_ScreenShot_f: Couldn't create %s\n", imagename);
 
 	free (buffer);
 }
@@ -924,6 +970,8 @@ int SCR_ModalMessage (const char *text, float timeout) //johnfitz -- timeout
 	} while (lastchar != 'y' && lastchar != 'Y' &&
 		 lastchar != 'n' && lastchar != 'N' &&
 		 lastkey != K_ESCAPE &&
+		 lastkey != K_ABUTTON &&
+		 lastkey != K_BBUTTON &&
 		 time2 <= time1);
 	Key_EndInputGrab ();
 
@@ -934,7 +982,7 @@ int SCR_ModalMessage (const char *text, float timeout) //johnfitz -- timeout
 		return false;
 	//johnfitz
 
-	return (lastchar == 'y' || lastchar == 'Y');
+	return (lastchar == 'y' || lastchar == 'Y' || lastkey == K_ABUTTON);
 }
 
 
