@@ -50,7 +50,7 @@ static cvar_t in_debugkeys = {"in_debugkeys", "0", CVAR_NONE};
 
 // SDL2 Game Controller cvars
 cvar_t	joy_deadzone = { "joy_deadzone", "0.175", CVAR_ARCHIVE };
-cvar_t	joy_deadzone_trigger = { "joy_deadzone_trigger", "0.001", CVAR_ARCHIVE };
+cvar_t	joy_deadzone_trigger = { "joy_deadzone_trigger", "0.2", CVAR_ARCHIVE };
 cvar_t	joy_sensitivity_yaw = { "joy_sensitivity_yaw", "300", CVAR_ARCHIVE };
 cvar_t	joy_sensitivity_pitch = { "joy_sensitivity_pitch", "150", CVAR_ARCHIVE };
 cvar_t	joy_invert = { "joy_invert", "0", CVAR_ARCHIVE };
@@ -81,7 +81,7 @@ static int buttonremap[] =
 /* total accumulated mouse movement since last frame */
 static int	total_dx, total_dy = 0;
 
-static int IN_FilterMouseEvents (const SDL_Event *event)
+static int SDLCALL IN_FilterMouseEvents (const SDL_Event *event)
 {
 	switch (event->type)
 	{
@@ -95,13 +95,13 @@ static int IN_FilterMouseEvents (const SDL_Event *event)
 }
 
 #if defined(USE_SDL2)
-static int IN_SDL2_FilterMouseEvents (void *userdata, SDL_Event *event)
+static int SDLCALL IN_SDL2_FilterMouseEvents (void *userdata, SDL_Event *event)
 {
 	return IN_FilterMouseEvents (event);
 }
 #endif
 
-static void IN_BeginIgnoringMouseEvents()
+static void IN_BeginIgnoringMouseEvents(void)
 {
 #if defined(USE_SDL2)
 	SDL_EventFilter currentFilter = NULL;
@@ -116,7 +116,7 @@ static void IN_BeginIgnoringMouseEvents()
 #endif
 }
 
-static void IN_EndIgnoringMouseEvents()
+static void IN_EndIgnoringMouseEvents(void)
 {
 #if defined(USE_SDL2)
 	SDL_EventFilter currentFilter;
@@ -416,6 +416,11 @@ static joyaxisstate_t joy_axisstate;
 static double joy_buttontimer[SDL_CONTROLLER_BUTTON_MAX];
 static double joy_emulatedkeytimer[10];
 
+#ifdef __WATCOMC__ /* OW1.9 doesn't have powf() / sqrtf() */
+#define powf pow
+#define sqrtf sqrt
+#endif
+
 /*
 ================
 IN_AxisMagnitude
@@ -669,7 +674,7 @@ void IN_JoyMove (usercmd_t *cmd)
 	moveEased = IN_ApplyMoveEasing(moveDeadzone);
 	lookEased = IN_ApplyLookEasing(lookDeadzone, joy_exponent.value);
 	
-	if (in_speed.state & 1)
+	if ((in_speed.state & 1) ^ (cl_alwaysrun.value != 0.0))
 		speed = cl_movespeedkey.value;
 	else
 		speed = 1;
@@ -748,11 +753,21 @@ void IN_UpdateInputMode (void)
 		textmode = want_textmode;
 #if !defined(USE_SDL2)
 		SDL_EnableUNICODE(textmode);
+		if (in_debugkeys.value)
+			Con_Printf("SDL_EnableUNICODE %d time: %g\n", textmode, Sys_DoubleTime());
 #else
 		if (textmode)
+		{
 			SDL_StartTextInput();
+			if (in_debugkeys.value)
+				Con_Printf("SDL_StartTextInput time: %g\n", Sys_DoubleTime());
+		}
 		else
+		{
 			SDL_StopTextInput();
+			if (in_debugkeys.value)
+				Con_Printf("SDL_StopTextInput time: %g\n", Sys_DoubleTime());
+		}
 #endif
 	}
 }
@@ -959,7 +974,7 @@ static inline int IN_SDL2_ScancodeToQuakeKey(SDL_Scancode scancode)
 #if defined(USE_SDL2)
 static void IN_DebugTextEvent(SDL_Event *event)
 {
-	Con_Printf ("SDL_TEXTINPUT '%s'\n", event->text.text);
+	Con_Printf ("SDL_TEXTINPUT '%s' time: %g\n", event->text.text, Sys_DoubleTime());
 }
 #endif
 
@@ -967,15 +982,17 @@ static void IN_DebugKeyEvent(SDL_Event *event)
 {
 	const char *eventtype = (event->key.state == SDL_PRESSED) ? "SDL_KEYDOWN" : "SDL_KEYUP";
 #if defined(USE_SDL2)
-	Con_Printf ("%s scancode: '%s' keycode: '%s'\n",
+	Con_Printf ("%s scancode: '%s' keycode: '%s' time: %g\n",
 		eventtype,
 		SDL_GetScancodeName(event->key.keysym.scancode),
-		SDL_GetKeyName(event->key.keysym.sym));
+		SDL_GetKeyName(event->key.keysym.sym),
+		Sys_DoubleTime());
 #else
-	Con_Printf ("%s sym: '%s' unicode: %04x\n",
+	Con_Printf ("%s sym: '%s' unicode: %04x time: %g\n",
 		eventtype,
 		SDL_GetKeyName(event->key.keysym.sym),
-		(int)event->key.keysym.unicode);
+		(int)event->key.keysym.unicode,
+		Sys_DoubleTime());
 #endif
 }
 
