@@ -65,7 +65,13 @@ static char *PF_VarString (int	first)
 		}
 	}
 	if (s > 255)
-		Con_DWarning("PF_VarString: %i characters exceeds standard limit of 255.\n", (int) s);
+	{
+		if (!dev_overflows.varstring || dev_overflows.varstring + CONSOLE_RESPAM_TIME < realtime)
+		{
+			Con_DWarning("PF_VarString: %i characters exceeds standard limit of 255 (max = %d).\n", (int) s, (int)(sizeof(out) - 1));
+			dev_overflows.varstring = realtime;
+		}
+	}
 	return out;
 }
 
@@ -729,7 +735,8 @@ static void PF_checkpos (void)
 
 //============================================================================
 
-static byte	checkpvs[MAX_MAP_LEAFS/8];
+static byte	*checkpvs;	//ericw -- changed to malloc
+static int	checkpvs_capacity;
 
 static int PF_newcheckclient (int check)
 {
@@ -738,6 +745,7 @@ static int PF_newcheckclient (int check)
 	edict_t	*ent;
 	mleaf_t	*leaf;
 	vec3_t	org;
+	int	pvsbytes;
 
 // cycle to the next one
 
@@ -776,7 +784,16 @@ static int PF_newcheckclient (int check)
 	VectorAdd (ent->v.origin, ent->v.view_ofs, org);
 	leaf = Mod_PointInLeaf (org, sv.worldmodel);
 	pvs = Mod_LeafPVS (leaf, sv.worldmodel);
-	memcpy (checkpvs, pvs, (sv.worldmodel->numleafs+7)>>3 );
+	
+	pvsbytes = (sv.worldmodel->numleafs+7)>>3;
+	if (checkpvs == NULL || pvsbytes > checkpvs_capacity)
+	{
+		checkpvs_capacity = pvsbytes;
+		checkpvs = (byte *) realloc (checkpvs, checkpvs_capacity);
+		if (!checkpvs)
+			Sys_Error ("PF_newcheckclient: realloc() failed on %d bytes", checkpvs_capacity);
+	}
+	memcpy (checkpvs, pvs, pvsbytes);
 
 	return i;
 }
@@ -1222,6 +1239,13 @@ static void PF_lightstyle (void)
 	style = G_FLOAT(OFS_PARM0);
 	val = G_STRING(OFS_PARM1);
 
+// bounds check to avoid clobbering sv struct
+	if (style < 0 || style >= MAX_LIGHTSTYLES)
+	{
+		Con_DWarning("PF_lightstyle: invalid style %d\n", style);
+		return;
+	}
+
 // change the string in sv
 	sv.lightstyles[style] = val;
 
