@@ -211,8 +211,8 @@ void SV_AddUpdates (void);
 void SV_ClientThink (void);
 void SV_AddClientToServer (struct qsocket_s	*ret);
 
-void SV_ClientPrintf (const char *fmt, ...) __attribute__((__format__(__printf__,1,2)));
-void SV_BroadcastPrintf (const char *fmt, ...) __attribute__((__format__(__printf__,1,2)));
+void SV_ClientPrintf (const char *fmt, ...) FUNC_PRINTF(1,2);
+void SV_BroadcastPrintf (const char *fmt, ...) FUNC_PRINTF(1,2);
 
 void SV_Physics (void);
 
